#ifndef IMAGE_INC
#define IMAGE_INC

#include "Sequence.h"
#include "vision.h"
#include "Color.h"

namespace Torch {

class ImageDiskXFile;

/** This class is designed to handle images

    An image is stored as a sequence of one frame in interleaved format (ex: rgbrgb...rgb).
    Consequently, #n_frames = 1# and #frames[0]# contains the image.
    
    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 1.0
*/
class Image : public Sequence
{
   	/// unique image identifier
	int image_id;

protected:

	/// memory allocation/re-allocation
	void imageRealloc();
	
public:
   	/**@name data and pixmap fields */
        //@{
	/// The image in real idem as #frames[0]#
	real *data;

	/// Also the image but in bytes
	unsigned char *pixmap;
        //@}
	
   	/**@name image size fields */
        //@{
	/// width of the image
	int width;

	/// height of the image
	int height;
	
	/// Number of pixels of the image (in use)
	int image_size;

	/// Real number of pixels allocated
	int real_image_size;

	/// Number of color planes (channels)
	int n_planes;
	//@}

   	/**@name other fields */
        //@{
	/// Coding format. Could be "rgb", "yuv", "gray" ...
	char *coding;

	/// verbose
	bool verbose;
	//@}
	
	//-----

   	/**@name constructors and destructor */
        //@{
	/// Creates an empty image 
    	Image();

	/** Creates a black image.
	
	    @param width_ is the width of the image
	    @param height_ is the height of the image
	    @param n_planes_ is the number of color components of the image (3 for RGB, 1 for gray)
	*/
    	Image(int width_, int height_, int n_planes_);

	///
	virtual ~Image();
	//@}
	
   	/**@name copy methods */
        //@{
	/** Copy from an image (in bytes).
	
	    @param width_ is the width of the image to copy
	    @param height_ is the height of the image to copy
	    @param pixmap_ is the image (in bytes) to copy into pixmap and data
	    @param coding_ is the coding of the #pixmap_# to copy
	*/
	virtual bool copyFrom(int width_, int height_, unsigned char *pixmap_, const char *coding_) = 0;

	/** Copy from an image (in real).
	
	    @param width_ is the width of the image to copy
	    @param height_ is the height of the image to copy
	    @param data_ is the image (in real) to copy into pixmap and data
	    @param coding_ is the coding of the #data_# to copy
	    @param max_ is the range of pixel values in #data_#
	*/
	virtual bool copyFrom(int width_, int height_, real *data_, const char *coding_, real max_ = 255.0) = 0;

	/** Copy an image from another image.
	
	    @param image_ is the image to copy
	*/
	virtual bool copyFrom(Image *image_) = 0;
	//@}
	
   	/**@name synchronization methods */
        //@{
	/// update the field #data# from the field #pixmap#
	virtual void updateDataFromPixmap();

	/// update the field #pixmap# from the field #data#
	virtual void updatePixmapFromData();
	//@}

   	/**@name load/save methods */
        //@{
	/** save the image using an ImageDiskXFile.
	
	    @param file is the ImageDiskXFile to use to save the image
	*/
	virtual bool saveImageXFile(ImageDiskXFile *file);

	/** load the image using an ImageDiskXFile.
	
	    @param file is the ImageDiskXFile to use to load the image
	*/
	virtual bool loadImageXFile(ImageDiskXFile *file);

	/// save the image
	virtual void saveXFile(XFile *file);

	/// load the image
	virtual void loadXFile(XFile *file);
	//@}

   	/**@name drawing methods */
        //@{
	/** draw a pixel in the image.
	
	    @param x_ is the #x# coordinate of the pixel to draw
	    @param y_ is the #y# coordinate of the pixel to draw
	    @param c is the color to draw
	*/
	virtual void drawpixel(int x_, int y_, Color c) = 0;

	/** draw a line P1-P2 in the image.
	
	    @param x1 is the #x# coordinate of #P1#
	    @param y1 is the #y# coordinate of #P1#
	    @param x2 is the #x# coordinate of #P2#
	    @param y2 is the #y# coordinate of #P2#
	    @param c is the color to draw
	*/
	void drawline(int x1, int y1, int x2, int y2, Color c);
	
	/** draw a line P1-P2 in the image.
	
	    @param x1 is the #x# coordinate of #P1#
	    @param y1 is the #y# coordinate of #P1#
	    @param x2 is the #x# coordinate of #P2#
	    @param y2 is the #y# coordinate of #P2#
	    @param c is the color to draw
	*/
	void drawline(double x1, double y1, double x2, double y2, Color c);
	//@}

   	/**@name other methods */
        //@{
	/// setting image id
	void setImageId(int image_id_) { image_id = image_id_; };

	/// getting image id
	int getImageId() { return image_id; };
	//@}
};


}

#endif
