#include "ipColorCube.h"
#include "XFile.h"
#include "Convert.h"

namespace Torch {

ipColorCube::ipColorCube(int width_, int height_, const char *coding_, real background_) : ipCore(coding_)
{
	if((coding_id != 1) && (coding_id != 2))
		error("ipColorCube() handles only RGB or YUV images.");

	width = width_;
        height = height_;
	background = background_;

	cube = NULL;
	
	seq_out = new(allocator) Sequence(1, width*height*n_planes);
	binaryimage = (unsigned char *)allocator->alloc(sizeof(unsigned char)*width*height);
	realbinaryimage = (real *)allocator->alloc(sizeof(real)*width*height);
	for(int i = 0; i < width * height ; i++)
	{
	   	seq_out->frames[0][n_planes*i] = 0.0;
	   	seq_out->frames[0][n_planes*i+1] = 0.0;
	   	seq_out->frames[0][n_planes*i+2] = 0.0;
		binaryimage[i] = 0;
		realbinaryimage[i] = 0.0;
	}
}

void ipColorCube::process(Sequence *seq_in)
{
   	real *data_ = seq_in->frames[0];
				
	n_pixels_filtered = 0;

	switch(coding_id)
	{
	case 1:
		// 
		for(int i = 0 ; i < width*height ; i++)
      		{
		   	int base = 3*i;
			unsigned char r, g, b;
			unsigned char y, u, v;
			int Y, U, V;
			unsigned char *pcube;
			
	        	r = (int) data_[base];
	        	g = (int) data_[base+1];
	        	b = (int) data_[base+2];

			//print("R=%03d G=%03d B=%03d", r, g, b);
			
			Y = GET_Y_FROM_RGB(r, g, b);
			U = GET_U_FROM_RGB(r, g, b);
			V = GET_V_FROM_RGB(r, g, b);
	
			y = (Y * 219) / 256 + 16; 
			u = (U * 224) /256 + 128;
			v = (V * 224) / 256 + 128;
		
			//print("-> Y=%03d U=%03d V=%03d\n", y, u, v);

			pcube = getcube(y, u, v);
			binaryimage[i] = *pcube;
			realbinaryimage[i] = binaryimage[i];
				
			//pcube = cube + (u >> ShiftUV) + ((v >> ShiftUV) << MUV);			 
			//binaryimage[i] = pcube[((y >> ShiftY) << MY)];

			if(binaryimage[i] == 1)
			{
				seq_out->frames[0][base] = r;
				seq_out->frames[0][base+1] = g;
				seq_out->frames[0][base+2] = b;

				n_pixels_filtered++;
			}
			else
			{
				seq_out->frames[0][base] = background;
				seq_out->frames[0][base+1] = background;
				seq_out->frames[0][base+2] = background;
			}
	      	}
		
		break;
	case 2:
		// 
		for(int i = 0 ; i < width*height ; i++)
      		{
		   	int base = 3*i;
			unsigned char y, u, v;
			unsigned char y_, u_, v_;
			int Y, U, V;
			unsigned char *pcube;
			
	        	y = (int) data_[base];
	        	u = (int) data_[base+1];
	        	v = (int) data_[base+2];

			Y = ICCIRY(y);
        	        U = ICCIRUV(u - 128);
                	V = ICCIRUV(v - 128);	
			
			y_ = (Y * 219) / 256 + 16; 
			u_ = (U * 224) /256 + 128;
			v_ = (V * 224) / 256 + 128;
		
			pcube = getcube(y_, u_, v_);
			binaryimage[i] = *pcube;
			realbinaryimage[i] =  binaryimage[i];

			//pcube = cube + (u_ >> ShiftUV) + ((v_ >> ShiftUV) << MUV);			 
			//binaryimage[i] = pcube[((y_ >> ShiftY) << MY)];

			if(binaryimage[i] == 1)
			{
				seq_out->frames[0][base] = y;
				seq_out->frames[0][base+1] = u;
				seq_out->frames[0][base+2] = v;

				n_pixels_filtered++;
			}
			else
			{
				seq_out->frames[0][base] = background;
				seq_out->frames[0][base+1] = background;
				seq_out->frames[0][base+2] = background;
			}
	      	}

		break;
	}
}

void ipColorCube::loadXFile(XFile *file)
{
  	unsigned int shift_uv;
  	unsigned int shift_y;
  	unsigned int size_uv;
  	unsigned int size_y;
	
	file->read(&size_uv, sizeof(unsigned int), 1);	
	file->read(&shift_uv, sizeof(unsigned int), 1);	
	file->read(&size_y, sizeof(unsigned int), 1);	
	file->read(&shift_y, sizeof(unsigned int), 1);	

   	ShiftUV = shift_uv;
   	ShiftY = shift_y;
   	DimUV = (int) size_uv;
   	DimY = (int) size_y;	

	// on linux
   	MUV = (int) (log((double) DimUV) / log((double) 2.0)) + 1;
	// on solaris
   	//MUV = (int) log2((double) DimUV);
	
   	MY = 2 * MUV;

   	int size = (size_uv * size_uv * size_y);
	if(cube != NULL) allocator->free(cube);
   	cube = (unsigned char *)allocator->alloc(sizeof(unsigned char) * size);	

	file->read(cube, DimY * DimUV * DimUV, 1);

	if(verbose) 
	{
		print("Colorcube:\n");
		print("   Dim Y = %d\n", DimY);
		print("   Shift Y = %d\n", ShiftY);
		print("   Dim UV = %d\n", DimUV);
		print("   Shift UV = %d\n", ShiftUV);
		print("   MY = %d\n", MY);
		print("   MUV = %d\n", MUV);
	}
}

void ipColorCube::saveXFile(XFile *file)
{
  	unsigned int shift_uv = ShiftUV;
  	unsigned int shift_y  = ShiftY;
  	unsigned int size_uv  = DimUV;
  	unsigned int size_y   = DimY;
	
	file->write(&size_uv, sizeof(unsigned int), 1);	
	file->write(&shift_uv, sizeof(unsigned int), 1);	
	file->write(&size_y, sizeof(unsigned int), 1);	
	file->write(&shift_y, sizeof(unsigned int), 1);	

	file->write(cube, DimY * DimUV * DimUV, 1);
}
	
unsigned char *ipColorCube::getcube(unsigned char y, unsigned char u, unsigned char v)
{
	unsigned char *pcube = cube + (u >> ShiftUV) + ((v >> ShiftUV) << MUV);

	return &pcube[((y >> ShiftY) << MY)];
}
	
void ipColorCube::add(int n_inputs_, real *data_)
{
	switch(coding_id)
	{
	case 1:
		// 
		for(int i = 0 ; i < n_inputs_ ; i++)
      		{
		   	int base = 3*i;
			unsigned char r, g, b;
			unsigned char y, u, v;
			int Y, U, V;
			unsigned char *pcube;
			
	        	r = (int) data_[base];
	        	g = (int) data_[base+1];
	        	b = (int) data_[base+2];
			
			Y = GET_Y_FROM_RGB(r, g, b);
			U = GET_U_FROM_RGB(r, g, b);
			V = GET_V_FROM_RGB(r, g, b);
	
			y = (Y * 219) / 256 + 16; 
			u = (U * 224) /256 + 128;
			v = (V * 224) / 256 + 128;
		
			pcube = getcube(y, u, v);
			*pcube = 1;
	      	}
		
		break;
	case 2:
		// 
		for(int i = 0 ; i < n_inputs_ ; i++)
      		{
		   	int base = 3*i;
			unsigned char y, u, v;
			unsigned char y_, u_, v_;
			int Y, U, V;
			unsigned char *pcube;
			
	        	y = (int) data_[base];
	        	u = (int) data_[base+1];
	        	v = (int) data_[base+2];

			Y = ICCIRY(y);
        	        U = ICCIRUV(u - 128);
                	V = ICCIRUV(v - 128);	
			
			y_ = (Y * 219) / 256 + 16; 
			u_ = (U * 224) /256 + 128;
			v_ = (V * 224) / 256 + 128;
		
			pcube = getcube(y_, u_, v_);
			*pcube = 1;
	      	}

		break;
	}
}
	
void ipColorCube::remove(int n_inputs_, real *data_)
{
	switch(coding_id)
	{
	case 1:
		// 
		for(int i = 0 ; i < n_inputs_ ; i++)
      		{
		   	int base = 3*i;
			unsigned char r, g, b;
			unsigned char y, u, v;
			int Y, U, V;
			unsigned char *pcube;
			
	        	r = (int) data_[base];
	        	g = (int) data_[base+1];
	        	b = (int) data_[base+2];
			
			Y = GET_Y_FROM_RGB(r, g, b);
			U = GET_U_FROM_RGB(r, g, b);
			V = GET_V_FROM_RGB(r, g, b);
	
			y = (Y * 219) / 256 + 16; 
			u = (U * 224) /256 + 128;
			v = (V * 224) / 256 + 128;
		
			pcube = getcube(y, u, v);
			*pcube = 0;
	      	}
		
		break;
	case 2:
		// 
		for(int i = 0 ; i < n_inputs_ ; i++)
      		{
		   	int base = 3*i;
			unsigned char y, u, v;
			unsigned char y_, u_, v_;
			int Y, U, V;
			unsigned char *pcube;
			
	        	y = (int) data_[base];
	        	u = (int) data_[base+1];
	        	v = (int) data_[base+2];

			Y = ICCIRY(y);
        	        U = ICCIRUV(u - 128);
                	V = ICCIRUV(v - 128);	
			
			y_ = (Y * 219) / 256 + 16; 
			u_ = (U * 224) /256 + 128;
			v_ = (V * 224) / 256 + 128;
		
			pcube = getcube(y_, u_, v_);
			*pcube = 0;
	      	}

		break;
	}
}

ipColorCube::~ipColorCube()
{
}

}

