#ifndef IPGABOR2D_INC
#define IPGABOR2D_INC

#include "ipCore.h"
#include "Mat.h"
#include "Mat_operations.h"
#include "matrix_addons.h"
#include "ipFFT2D.h"

namespace Torch {

/** This class is designed to compute a 2D Gabor filter

    Thanks to Souheil Ben Yacoub for the original source code.

    \begin{verbatim}

      seq_in->frames[0]        +--------------+
	       +----+          |              |
	       |    |   ---->  |  ipGabor2D   | ----> seq_out->frames[0] (Gabor real part)
	       |    |          |              |       seq_out->frames[1] (Gabor imag part)
	       +----+          +--------------+
               (image)
	 
    \end{verbatim}
    
    @author Sebastien Marcel (marcel@idiap.ch)
    
    @version 2.0
    \Date
    @since 2.0
*/
class ipGabor2D : public ipCore
{
   	// border size of the Gabor filter
   	int side;

	// the Gabor filter in the FT domain
	Mat *G_real;
	Mat *G_imag;

	// the image in the FT domain
	Mat *I_real;
	Mat *I_imag;
	
	// the filtered image in the FT domain
	Mat *F_real;
	Mat *F_imag;

	// temp image in the FT domain
	Mat *tmp_r;
	Mat *tmp_i;

	// temp sequence
	Sequence *seq_;

public:
	/// scale of the filter
	int s;

	/// orientation of the filter
	int n;

   	/// real part of the filter
	Mat *Gr;

	/// imaginary part of the filter
	Mat *Gi;

	/// width of the filter
	int width;

	/// height of the filter
	int height;
   
	/// FFT
	ipFFT2D *fft;

	/// IFFT
	ipFFT2D *ifft;

	/// real width of the image
	int real_width;

	/// real width of the image
	int real_height;


	//----
	
	///
	ipGabor2D(int width_, int height_, int s_, int n_, real Ul, real Uh, int scale, int orientation, int verbose_ = 0);

	///
    	virtual ~ipGabor2D();

	///
    	virtual void process(Sequence *seq_in);

	/// to perform the filtering in the FT domain
	void setFFT(ipFFT2D *fft_, ipFFT2D *ifft_, int real_width_, int real_height_);
};

}

#endif
