#ifndef IPSMOOTH_GAUSSIAN3_INC
#define IPSMOOTH_GAUSSIAN3_INC

#include "ipCore.h"

namespace Torch {

/** This class is designed to smooth an image by convolving a 3x3 Gaussian filter

    \TEX{
    	\begin{eqnarray}
    	O(x,y) = G(x,y) * I(x,y) \\
    	G(x,y) = \frac{exp(x^2 + y^2)}{\sigma}
	\end{eqnarray}
	
	\begin{itemize} 
	\item $I(x,y)$ is the input image, 
	\item $O(x,y)$ is the output image, 
	\item $I,O \in {{\rm I \kern -0.2em R}}^w \times {{\rm I \kern -0.2em R}}^h$,
	\item $w$ is the width of the image and $h$ is the height of the image,
	\item $\sigma$ is the variance of the kernel,
	\item * is the convolution operator.
	\end{itemize}
    }
  
    In output seq_out->frames[0] contains the smoothed image.

    @author Sebastien Marcel (marcel@idiap.ch)
    @version 2.0
    \Date
    @since 2.0
*/
class ipSmoothGaussian3 : public ipCore
{
   	// width of the input image
	int width;

	// height of the input image
	int height;

	// variance of the gaussian kernel
   	real sigma;

	// the pre-computed kernel
	real *kernel;

	// compute the kernel
	void init_(void);

public:
	/** constructor
	    
	    @param width_ is the #width# of the input image
	    @param height_ is the #height# of the input image
	    @param coding_ is the #coding# format of the input image
	    @param sigma_ is the variance of the Gaussian kernel
	*/
    	ipSmoothGaussian3(int width_, int height_, char *coding_, real sigma_ = 0.25);

	///
	virtual ~ipSmoothGaussian3();

	///
	virtual void process(Sequence *seq_in);
};

}

#endif
