#include "RealStumpTrainer.h"

namespace Torch {

RealStumpTrainer::RealStumpTrainer(RealStumpMachine *real_machine_) : Trainer(real_machine_)
{
	addBOption("verbose", &verbose, false, "verbose");

   	real_machine = real_machine_;
	n_bins = real_machine->n_bins;

	bestFeature = 0;
	min_feat = INF;
	max_feat = -INF;
	lut = (real *)allocator->alloc(sizeof(real)*n_bins);

	n_pos = 0;
	n_neg = 0;
	best_score = -INF;

	histoPos = NULL; 
	histoNeg = NULL;
	
	features = NULL;
	feature_index = NULL;
	targets = NULL;
}

void RealStumpTrainer::train(DataSet *data_, MeasurerList *measurers)  
{	
	bestFeature = 0;
	min_feat = INF;
	max_feat = -INF;

	best_score = -INF;
	
	int n_examples = data_->n_examples;

	histoPos = (int *)allocator->alloc(sizeof(int)*n_bins);
	histoNeg = (int *)allocator->alloc(sizeof(int)*n_bins);
	
	features = (real *)allocator->alloc(sizeof(real)*n_examples);
	feature_index = (int *)allocator->alloc(sizeof(int)*n_examples);
	targets = (real *)allocator->alloc(sizeof(real)*n_examples);

	// computes n_pos, n_neg and fills targets
	n_pos = 0, n_neg = 0;
	for(int i = 0; i < n_examples; i++)
	{
		data_->setExample(i);
		targets[i] = data_->targets->frames[0][0];
		if(targets[i] == 1) n_pos++;
		else n_neg++;
	}
	if(n_pos + n_neg != n_examples) error("RealStumpTrainer: wrong number of examples.");
	message("n_positives: %d   n_negatives: %d", n_pos, n_neg);
	
	
	for(int f = 0; f < real_machine->n_features; f++)
	{
		//print("\rn_features: %d / %d     %2.4f             ", feat, real_machine->n_features, best_score);
		computeLogLikelihood(data_, f);
	}

	message("\n RealStumpTrainer:  feature -> %d  min -> %2.4f  max -> %2.4f", bestFeature, min_feat, max_feat);
	real_machine->setParams(bestFeature, min_feat, max_feat, lut);
	
	message("best score: %2.4f", best_score);


	//verbose = true;
	computeLogLikelihood(data_, bestFeature);

	
	allocator->free(histoPos);
	allocator->free(histoNeg);
	
	allocator->free(features);
	allocator->free(feature_index);
	allocator->free(targets);
}

void RealStumpTrainer::computeLogLikelihood(DataSet *data_, int feature_id)
{
	// 1) calcule le tableau de features et calcul quelques statistiques (min, max, mean, ...)
	//----------------------------------------------------------------------------------------
	int n_examples = data_->n_examples;

	real mean = 0.0;
	real stdv = 0.0;

	real sum_pos = 0.0, sum_neg = 0.0;
	real min_ = INF, max_ = -INF;
	
	for(int i = 0; i < n_examples; i++)
	{
		data_->setExample(i);

		features[i] = real_machine->getFeature(feature_id, data_->inputs);

		if(targets[i] == 1) sum_pos += features[i];
		else sum_neg += features[i];
	
		if(features[i] < min_) min_ = features[i];
		if(features[i] > max_) max_ = features[i];

		mean += features[i]; 
		stdv += features[i] * features[i];
	}
	sum_pos /= n_pos;
	sum_neg /= n_neg;
	
	mean /= n_examples;
	stdv /= n_examples;
	stdv -= mean * mean;
	stdv = sqrt(stdv);
	if(verbose) message("min: %2.4f  max: %2.4f  mean: %2.4f  stdv: %2.4f", min_, max_, mean, stdv);


	// 2) remplit les deux histogrammes
	//----------------------------------------------------------------------------------------
	for(int j = 0; j < n_bins; j++)
	{
		histoPos[j] = 0;
		histoNeg[j] = 0;
	}
	
	real sample = (max_ - min_)/n_bins;
	real shift = -min_;

	for(int i=0; i<n_examples; i++)
	{
		real value_ = features[i] + shift;

		int index = (int)(floor(value_/sample));
		if(index < 0) index = 0;
		if(index >= n_bins) index = n_bins-1;

		if(targets[i] == 1) histoPos[index]++;
		else histoNeg[index]++;

		feature_index[i] = index;
	}

	real inv_n_pos = 1./(n_pos);
	real inv_n_neg = 1./(n_neg);

	
/*	for(int j = 0; j < n_bins; j++)
	{
		// histogram normalization (to have densities)
		real pos_prob = (real)histoPos[j] * inv_n_pos;
		real neg_prob = (real)histoNeg[j] * inv_n_neg;

		real max_prob = ( (pos_prob > neg_prob) ? pos_prob : neg_prob);

		if(max_prob != 0) lut[j] = (pos_prob - neg_prob) / max_prob;
		else lut[j] = 0;
	}

*/
	// 3) vire 1% des donnees de chaque cote des histogrammes (bruits)
	//----------------------------------------------------------------------------------------
	int a_virer = n_pos / 100;
	if(verbose) message("a virer (positive patterns): %d", a_virer);
	int vires = 0;
	for(int j = 0; j < n_bins; j++)
	{
		vires += histoPos[j];
		if(vires > a_virer) break;
		else histoPos[j] = 0;
	}
	vires = 0;
	for(int j = n_bins-1; j >= 0; j--)
	{
		vires += histoPos[j];
		if(vires > a_virer) break;
		else histoPos[j] = 0;
	}
	// negative patterns
	a_virer = n_neg / 100;
	if(verbose) message("a virer (negative patterns): %d", a_virer);
	vires = 0;
	for(int j = 0; j < n_bins; j++)
	{
		vires += histoNeg[j];
                if(vires > a_virer) break;
                else histoNeg[j] = 0;
	}
	vires = 0;
	for(int j = n_bins-1; j >= 0; j--)
	{
		vires += histoNeg[j];
                if(vires > a_virer) break;
                else histoNeg[j] = 0;
	}
	
	// 4) calcule la look-up table (log likelihood ratio)
	//----------------------------------------------------------------------------------------
	real epsilon = 0.000001;
	for(int j = 0; j < n_bins; j++)
	{
		real pos_prob = (real)histoPos[j] * inv_n_pos;
		real neg_prob = (real)histoNeg[j] * inv_n_neg;

		lut[j] = 0.5 * log( (pos_prob + epsilon) / (neg_prob + epsilon) );
	}

	real lut_tmp[n_bins];
	for(int j = 0; j < n_bins; j++)
		lut_tmp[j] = lut[j];
	
	/*real max_bin = -INF, min_bin = INF;
	int max_index = -1, min_index = -1;
	for(int j = 0; j < n_bins; j++)
	{
		if(lut[j] > max_bin) { max_bin = lut[j]; max_index = j; }
		if(lut[j] < min_bin) { min_bin = lut[j]; min_index = j; }
	}
	
	if(sum_pos > sum_neg) // dist. face a droite..
	{
		for(int j = 0; j < min_index; j++) lut[j] = lut[min_index];
		for(int j = max_index+1; j < n_bins; j++) lut[j] = lut[max_index];
	}
	else
	{
		for(int j = 0; j < max_index; j++) lut[j] = lut[max_index];
		for(int j = min_index+1; j < n_bins; j++) lut[j] = lut[min_index];
	}*/
	
	real score = 0.0;
	real score2 = 0.0;
	for(int i = 0; i < n_examples; i++)
		//score += targets[i] * lut[feature_index[i]];
	{
		if(lut[feature_index[i]] >= 0)
			score += targets[i];
		else
			score -= targets[i];
	}
	
	for(int j = 0; j < n_bins; j++)
		score2 += (real)(histoPos[j] - histoNeg[j]) * lut[j] ;
			
	if(score > best_score)
	{
		best_score = score;
		bestFeature = feature_id;
		min_feat = min_;
		max_feat = max_;
	}

	/*
	if(verbose)
	{
		for(int j = 0; j < n_bins; j++)
			print("\t%d\t%d\t%2.4f\t%2.4f\tscore: %2.4f\n", histoPos[j], histoNeg[j], lut_tmp[j], lut[j], (histoPos[j] - histoNeg[j]) * lut[j]);
		message("score: %f  score2: %f", score, score2);
	}
	*/
}

RealStumpTrainer::~RealStumpTrainer()
{
}

}
